@@ -4,7 +4,7 @@
  *                                                                           *
  * This file is part of HDF5.  The full HDF5 copyright notice, including     *
  * terms governing use, modification, and redistribution, is contained in    *
- * the COPYING file, which can be found at the root of the source code       *
+ * the LICENSE file, which can be found at the root of the source code       *
  * distribution tree, or in https://www.hdfgroup.org/licenses.               *
  * If you do not have access to either file, you may request a copy from     *
  * help@hdfgroup.org.                                                        *
@@ -628,6 +628,7 @@ test_core(void)
     /* Create file image buffer */
     fd = HDopen(copied_filename, O_RDONLY);
     VERIFY(fd > 0, "open failed");
+    memset(&sb, 0, sizeof(h5_stat_t));
     ret = HDfstat(fd, &sb);
     VERIFY(ret == 0, "fstat failed");
     size       = (size_t)sb.st_size;
@@ -648,7 +649,8 @@ test_core(void)
         FAIL_STACK_ERROR;
 
     /* Release resources */
-    h5_clean_files(FILENAME, fapl);
+    h5_delete_all_test_files(FILENAME, fapl);
+    H5Pclose(fapl);
     free(udata);
     free(file_image);
     HDremove(copied_filename);
@@ -668,15 +670,9 @@ error:
  *
  ******************************************************************************
  */
-/* Disable warning for "format not a string literal" here -QAK */
-/*
- *      This pragma only needs to surround the snprintf() calls with
- *      'member_file_name' in the code below, but early (4.4.7, at least) gcc only
- *      allows diagnostic pragmas to be toggled outside of functions.
- */
-H5_GCC_CLANG_DIAG_OFF("format-nonliteral")
 static int
-test_get_file_image(const char *test_banner, const int file_name_num, hid_t fapl, bool user)
+test_get_file_image(const char *test_banner, const int file_name_num, hid_t fapl, bool user,
+                    H5F_libver_t format)
 {
     char      file_name[1024] = "\0";
     void     *insertion_ptr   = NULL;
@@ -705,6 +701,14 @@ test_get_file_image(const char *test_banner, const int file_name_num, hid_t fapl
 
     TESTING(test_banner);
 
+    memset(&stat_buf, 0, sizeof(h5_stat_t));
+
+    /* Set file format */
+    if (format >= H5F_LIBVER_EARLIEST) {
+        ret = H5Pset_libver_bounds(fapl, format, H5F_LIBVER_LATEST);
+        VERIFY(ret >= 0, "H5Pset_libver_bounds");
+    }
+
     /* set flag if we are dealing with a family file */
     driver = H5Pget_driver(fapl);
     VERIFY(driver >= 0, "H5Pget_driver(fapl) failed");
@@ -769,6 +773,7 @@ test_get_file_image(const char *test_banner, const int file_name_num, hid_t fapl
     err = H5Fclose(file_id);
     VERIFY(err == SUCCEED, "H5Fclose(file_id) failed.");
 
+    /* Read file from disk */
     if (is_family_file) {
         char    member_file_name[1024];
         ssize_t bytes_to_read;
@@ -783,7 +788,9 @@ test_get_file_image(const char *test_banner, const int file_name_num, hid_t fapl
         file_size = 0;
 
         do {
+            H5_WARN_FORMAT_NONLITERAL_OFF
             snprintf(member_file_name, (size_t)1024, file_name, i);
+            H5_WARN_FORMAT_NONLITERAL_ON
 
             /* get the size of the member file */
             result = HDstat(member_file_name, &stat_buf);
@@ -799,7 +806,7 @@ test_get_file_image(const char *test_banner, const int file_name_num, hid_t fapl
          * may be larger.  This is OK, as long as (in this specialized instance)
          * the remainder of the file is all '\0's.
          */
-        VERIFY(file_size >= image_size, "file size != image size.");
+        VERIFY(file_size >= image_size, "file size < image size.");
 
         /* allocate a buffer for the test file image */
         file_image_ptr = malloc((size_t)file_size);
@@ -811,7 +818,9 @@ test_get_file_image(const char *test_banner, const int file_name_num, hid_t fapl
 
         while (size_remaining > 0) {
             /* construct the member file name */
+            H5_WARN_FORMAT_NONLITERAL_OFF
             snprintf(member_file_name, 1024, file_name, i);
+            H5_WARN_FORMAT_NONLITERAL_ON
 
             /* open the test file using standard I/O calls */
             fd = HDopen(member_file_name, O_RDONLY);
@@ -871,7 +880,7 @@ test_get_file_image(const char *test_banner, const int file_name_num, hid_t fapl
             HDoff_t off;
 
             /* Position at userblock */
-            off = HDlseek(fd, (HDoff_t)USERBLOCK_SIZE, SEEK_SET);
+            off = HDlseek(fd, USERBLOCK_SIZE, SEEK_SET);
             VERIFY(off >= 0, "HDlseek() failed.");
         }
 
@@ -921,7 +930,8 @@ test_get_file_image(const char *test_banner, const int file_name_num, hid_t fapl
     VERIFY(err == SUCCEED, "H5Pclose(core_fapl_id) failed.");
 
     /* tidy up */
-    h5_clean_files(FILENAME2, fapl);
+    h5_delete_all_test_files(FILENAME2, fapl);
+    H5Pclose(fapl);
 
     /* discard the image buffer if it exists */
     if (image_ptr != NULL)
@@ -938,7 +948,6 @@ test_get_file_image(const char *test_banner, const int file_name_num, hid_t fapl
 error:
     return 1;
 } /* end test_get_file_image() */
-H5_GCC_CLANG_DIAG_ON("format-nonliteral")
 
 /******************************************************************************
  * Function:    test_get_file_image_error_rejection
@@ -1067,7 +1076,8 @@ test_get_file_image_error_rejection(void)
     VERIFY(err == SUCCEED, "H5Fclose(file_id) failed.");
 
     /* tidy up */
-    h5_clean_files(FILENAME2, fapl_id);
+    h5_delete_all_test_files(FILENAME2, fapl_id);
+    H5Pclose(fapl_id);
 
     /* discard the image buffer if it exists */
     if (image_ptr != NULL)
@@ -1171,7 +1181,8 @@ test_get_file_image_error_rejection(void)
     VERIFY(err == SUCCEED, "H5Fclose(2) failed.");
 
     /* tidy up */
-    h5_clean_files(FILENAME2, fapl_id);
+    h5_delete_all_test_files(FILENAME2, fapl_id);
+    H5Pclose(fapl_id);
 
     /************************** Test #3 **********************************/
     /* set up a split file driver test file, and try to get its image
@@ -1233,7 +1244,8 @@ test_get_file_image_error_rejection(void)
     VERIFY(err == SUCCEED, "H5Fclose(2) failed.");
 
     /* tidy up */
-    h5_clean_files(FILENAME2, fapl_id);
+    h5_delete_all_test_files(FILENAME2, fapl_id);
+    H5Pclose(fapl_id);
 
     /************************** Test #4 **********************************/
     /* set up a family file driver test file, and try to get its image
@@ -1293,7 +1305,8 @@ test_get_file_image_error_rejection(void)
     VERIFY(err == SUCCEED, "H5Fclose(2) failed.");
 
     /* tidy up */
-    h5_clean_files(FILENAME2, fapl_id);
+    h5_delete_all_test_files(FILENAME2, fapl_id);
+    H5Pclose(fapl_id);
 
     PASSED();
 
@@ -1306,12 +1319,13 @@ error:
 int
 main(void)
 {
-    int      errors = 0;
-    hid_t    fapl;
-    bool     driver_is_default_compatible;
-    unsigned user;
+    int          errors = 0;
+    hid_t        fapl;
+    bool         driver_is_default_compatible;
+    unsigned     user;
+    H5F_libver_t format;
 
-    h5_reset();
+    h5_test_init();
 
     printf("Testing File Image Functionality.\n");
 
@@ -1325,30 +1339,34 @@ main(void)
     }
 
     /* Perform tests with/without user block */
-    for (user = false; user <= true; user++) {
+    for (user = false; user <= true; user++)
 
-        /* test H5Fget_file_image() with sec2 driver */
-        fapl = H5Pcreate(H5P_FILE_ACCESS);
-        if (H5Pset_fapl_sec2(fapl) < 0)
-            errors++;
-        else
-            errors += test_get_file_image("H5Fget_file_image() with sec2 driver", 0, fapl, user);
+        /* Perform tests with different file format versions.  H5F_LIBVER_ERROR causes the test to use the
+         * default settings. */
+        for (format = H5F_LIBVER_ERROR; format <= H5F_LIBVER_LATEST; format++) {
 
-        /* test H5Fget_file_image() with stdio driver */
-        fapl = H5Pcreate(H5P_FILE_ACCESS);
-        if (H5Pset_fapl_stdio(fapl) < 0)
-            errors++;
-        else
-            errors += test_get_file_image("H5Fget_file_image() with stdio driver", 1, fapl, user);
+            /* test H5Fget_file_image() with sec2 driver */
+            fapl = H5Pcreate(H5P_FILE_ACCESS);
+            if (H5Pset_fapl_sec2(fapl) < 0)
+                errors++;
+            else
+                errors += test_get_file_image("H5Fget_file_image() with sec2 driver", 0, fapl, user, format);
 
-        /* test H5Fget_file_image() with core driver */
-        fapl = H5Pcreate(H5P_FILE_ACCESS);
-        if (H5Pset_fapl_core(fapl, (size_t)(64 * 1024), true) < 0)
-            errors++;
-        else
-            errors += test_get_file_image("H5Fget_file_image() with core driver", 2, fapl, user);
+            /* test H5Fget_file_image() with stdio driver */
+            fapl = H5Pcreate(H5P_FILE_ACCESS);
+            if (H5Pset_fapl_stdio(fapl) < 0)
+                errors++;
+            else
+                errors += test_get_file_image("H5Fget_file_image() with stdio driver", 1, fapl, user, format);
 
-    } /* end for */
+            /* test H5Fget_file_image() with core driver */
+            fapl = H5Pcreate(H5P_FILE_ACCESS);
+            if (H5Pset_fapl_core(fapl, (size_t)(64 * 1024), true) < 0)
+                errors++;
+            else
+                errors += test_get_file_image("H5Fget_file_image() with core driver", 2, fapl, user, format);
+
+        } /* end for */
 
 #if 0
     /* at present, H5Fget_file_image() rejects files opened with the
@@ -1372,7 +1390,7 @@ main(void)
 
     errors += test_get_file_image_error_rejection();
 
-    /* Restore the default error handler (set in h5_reset()) */
+    /* Restore the default error handler (set in h5_test_init()) */
     h5_restore_err();
 
     if (errors) {
